@@ -452,8 +452,8 @@ def eval_options(func: Callable[..., Any]) -> Callable[..., click.Context]:
     )
     @click.option(
         "--reasoning-effort",
-        type=click.Choice(["low", "medium", "high"]),
-        help="Constrains effort on reasoning for reasoning models (defaults to `medium`). Open AI o-series models only.",
+        type=click.Choice(["minimal", "low", "medium", "high"]),
+        help="Constrains effort on reasoning for reasoning models (defaults to `medium`). Open AI o-series and gpt-5 models only.",
         envvar="INSPECT_EVAL_REASONING_EFFORT",
     )
     @click.option(
@@ -921,13 +921,12 @@ def eval_exec(
     eval_sample_id = parse_sample_id(sample_id)
 
     # resolve sample_shuffle
-    eval_sample_shuffle = (
-        False
-        if sample_shuffle is None
-        else True
-        if sample_shuffle == -1
-        else sample_shuffle
-    )
+    if sample_shuffle == -1:
+        eval_sample_shuffle: Literal[True] | int | None = True
+    elif sample_shuffle == 0:
+        eval_sample_shuffle = None
+    else:
+        eval_sample_shuffle = sample_shuffle
 
     # resolve fail_on_error
     if no_fail_on_error is True:
