@@ -50,9 +50,11 @@ from inspect_ai._util.content import (
 from inspect_ai._util.content import (
     ContentAudio,
     ContentData,
+    ContentDocument,
     ContentImage,
     ContentReasoning,
     ContentText,
+    ContentToolUse,
     ContentVideo,
 )
 from inspect_ai._util.error import PrerequisiteError
@@ -215,6 +217,11 @@ class GoogleGenAIAPI(ModelAPI):
                         + "or the 'location' custom model arg (-M) when running against vertex."
                     )
 
+            # custom base_url
+            self.base_url = model_base_url(
+                self.base_url, ["GOOGLE_VERTEX_BASE_URL", "VERTEX_BASE_URL"]
+            )
+
         # normal google endpoint
         else:
             # read api key from env
@@ -240,14 +247,21 @@ class GoogleGenAIAPI(ModelAPI):
         tool_choice: ToolChoice,
         config: GenerateConfig,
     ) -> ModelOutput | tuple[ModelOutput | Exception, ModelCall]:
+        # http options
+        http_options = HttpOptions(
+            base_url=self.base_url,
+            api_version=self.api_version,
+        )
+
+        # apply timeout if specified
+        if config.timeout:
+            http_options.timeout = config.timeout * 1000
+
         # create client
         client = Client(
             vertexai=self.is_vertex(),
             api_key=self.api_key,
-            http_options={
-                "base_url": self.base_url,
-                "api_version": self.api_version,
-            },
+            http_options=http_options,
             **self.model_args,
         )
 
@@ -595,14 +609,16 @@ async def content_part(client: Client, content: InspectContent | str) -> Part:
     elif isinstance(content, ContentReasoning):
         return Part(text=content.reasoning or NO_CONTENT, thought=True)
     elif isinstance(content, ContentData):
-        assert False, "Google provider should never encounter ContentData"
+        raise RuntimeError("Google provider should never encounter ContentData")
+    elif isinstance(content, ContentToolUse):
+        raise RuntimeError("Google provider should never encounter ContentToolUse")
     else:
         return await chat_content_to_part(client, content)
 
 
 async def chat_content_to_part(
     client: Client,
-    content: ContentImage | ContentAudio | ContentVideo,
+    content: ContentImage | ContentAudio | ContentVideo | ContentDocument,
 ) -> Part:
     if isinstance(content, ContentImage):
         content_bytes, mime_type = await file_as_data(content.image)
@@ -736,9 +752,11 @@ def completion_choice_from_candidate(
                 ContentText
                 | ContentReasoning
                 | ContentImage
+                | ContentToolUse
                 | ContentAudio
                 | ContentVideo
                 | ContentData
+                | ContentDocument
             ]
         ) = ""
     # content.parts can be None when the finish_reason is MALFORMED_FUNCTION_CALL
@@ -973,7 +991,7 @@ def str_to_harm_block_threshold(threshold: str) -> HarmBlockThreshold:
 
 
 async def file_for_content(
-    client: Client, content: ContentAudio | ContentVideo
+    client: Client, content: ContentAudio | ContentVideo | ContentDocument
 ) -> File:
     # helper to write trace messages
     def trace(message: str) -> None:
@@ -982,8 +1000,10 @@ async def file_for_content(
     # get the file bytes and compute sha256 hash
     if isinstance(content, ContentAudio):
         file = content.audio
-    else:
+    elif isinstance(content, ContentVideo):
         file = content.video
+    else:
+        file = content.document
     content_bytes, mime_type = await file_as_data(file)
     content_sha256 = hashlib.sha256(content_bytes).hexdigest()
     # we cache uploads for re-use, open the db where we track that
