@@ -65,6 +65,7 @@ class OpenAIAPI(ModelAPI):
         responses_store: Literal["auto"] | bool = "auto",
         service_tier: str | None = None,
         client_timeout: float | None = None,
+        background: bool | None = None,
         **model_args: Any,
     ) -> None:
         # extract azure service prefix from model name (other providers
@@ -76,8 +77,15 @@ class OpenAIAPI(ModelAPI):
         else:
             self.service = None
 
-        # extract user model arg if provided
-        self.user: str | NotGiven = model_args.pop("user", NOT_GIVEN)
+        # extract prompt_cache_key model arg if provided
+        self.prompt_cache_key: str | NotGiven = model_args.pop(
+            "prompt_cache_key", NOT_GIVEN
+        )
+
+        # extract safety_identifier model arg if provided
+        self.safety_identifier: str | NotGiven = model_args.pop(
+            "safety_identifier", NOT_GIVEN
+        )
 
         # call super
         super().__init__(
@@ -88,14 +96,23 @@ class OpenAIAPI(ModelAPI):
             config=config,
         )
 
+        # set background bit (automatically use background for deep research)
+        if background is None and self.is_deep_research():
+            background = True
+        self.background = background
+
         # is this a model we use responses api by default for?
         responses_preferred = (
-            self.is_o_series() and not self.is_o1_early()
-        ) or self.is_codex()
+            (self.is_o_series() and not self.is_o1_early())
+            or self.is_codex()
+            or self.is_gpt_5()
+        )
 
         # resolve whether we are forcing the responses api
-        self.responses_api = self.is_computer_use_preview() or (
-            responses_api if responses_api is not None else responses_preferred
+        self.responses_api = (
+            background
+            or self.is_computer_use_preview()
+            or (responses_api if responses_api is not None else responses_preferred)
         )
 
         # resolve whether we are using the responses store
@@ -186,7 +203,9 @@ class OpenAIAPI(ModelAPI):
                 api_version=api_version,
                 azure_endpoint=base_url,
                 http_client=http_client,
-                timeout=client_timeout if client_timeout is not None else NOT_GIVEN,
+                timeout=self.client_timeout
+                if self.client_timeout is not None
+                else NOT_GIVEN,
                 **model_args,
             )
         else:
@@ -194,7 +213,9 @@ class OpenAIAPI(ModelAPI):
                 api_key=self.api_key,
                 base_url=model_base_url(base_url, "OPENAI_BASE_URL"),
                 http_client=http_client,
-                timeout=client_timeout if client_timeout is not None else NOT_GIVEN,
+                timeout=self.client_timeout
+                if self.client_timeout is not None
+                else NOT_GIVEN,
                 **model_args,
             )
 
@@ -220,6 +241,13 @@ class OpenAIAPI(ModelAPI):
         else:
             return not self.is_gpt() and bool(re.search(r"o\d+", name))
 
+    def is_deep_research(self) -> bool:
+        return "deep-research" in self.service_model_name()
+
+    def is_gpt_5(self) -> bool:
+        name = self.service_model_name()
+        return "gpt-5" in name
+
     def is_o1(self) -> bool:
         name = self.service_model_name()
         return "o1" in name and not self.is_o1_early()
@@ -252,6 +280,10 @@ class OpenAIAPI(ModelAPI):
     def emulate_reasoning_history(self) -> bool:
         return not self.responses_api
 
+    @override
+    def supports_remote_mcp(self) -> bool:
+        return True
+
     @override
     def tool_result_images(self) -> bool:
         # computer_use_preview supports tool calls returning images
@@ -298,8 +330,10 @@ class OpenAIAPI(ModelAPI):
                 tools=tools,
                 tool_choice=tool_choice,
                 config=config,
+                background=self.background,
                 service_tier=self.service_tier,
-                user=self.user,
+                prompt_cache_key=self.prompt_cache_key,
+                safety_identifier=self.safety_identifier,
                 openai_api=self,
                 batcher=self._responses_batcher,
             )
@@ -312,7 +346,8 @@ class OpenAIAPI(ModelAPI):
                 tools=tools,
                 tool_choice=tool_choice,
                 config=config,
-                user=self.user,
+                prompt_cache_key=self.prompt_cache_key,
+                safety_identifier=self.safety_identifier,
                 openai_api=self,
                 batcher=self._completions_batcher,
             )
