@@ -1,10 +1,19 @@
 from logging import getLogger
 from typing import TYPE_CHECKING, Any
 
+import anyio
 from openai import AsyncAzureOpenAI, AsyncOpenAI, BadRequestError, NotGiven
 from openai._types import NOT_GIVEN
 from openai.types.responses import Response, ResponseFormatTextJSONSchemaConfigParam
+from tenacity import (
+    retry,
+    retry_if_exception,
+    stop_after_attempt,
+    stop_after_delay,
+    wait_exponential_jitter,
+)
 
+from inspect_ai._util.httpx import httpx_should_retry, log_httpx_retry_attempt
 from inspect_ai._util.logger import warn_once
 from inspect_ai.model._providers._openai_batch import OpenAIBatcher
 from inspect_ai.tool import ToolChoice, ToolInfo
@@ -40,11 +49,17 @@ async def generate_responses(
     tools: list[ToolInfo],
     tool_choice: ToolChoice,
     config: GenerateConfig,
+    background: bool | None,
     service_tier: str | None,
-    user: str | NotGiven,
+    prompt_cache_key: str | NotGiven,
+    safety_identifier: str | NotGiven,
     openai_api: "OpenAIAPI",
     batcher: OpenAIBatcher[Response] | None,
 ) -> ModelOutput | tuple[ModelOutput | Exception, ModelCall]:
+    # batch mode and background are incompatible
+    if batcher:
+        background = False
+
     # allocate request_id (so we can see it from ModelCall)
     request_id = http_hooks.start_request()
 
@@ -75,7 +90,7 @@ async def generate_responses(
         else NOT_GIVEN,
         truncation="auto" if openai_api.is_computer_use_preview() else NOT_GIVEN,
         extra_headers={HttpxHooks.REQUEST_ID_HEADER: request_id},
-        user=user,
+        background=background,
         **completion_params_responses(
             model_name,
             openai_api=openai_api,
@@ -84,6 +99,10 @@ async def generate_responses(
             tools=len(tools) > 0,
         ),
     )
+    if isinstance(prompt_cache_key, str):
+        request["prompt_cache_key"] = prompt_cache_key
+    if isinstance(safety_identifier, str):
+        request["safety_identifier"] = safety_identifier
 
     try:
         # generate response
@@ -96,6 +115,10 @@ async def generate_responses(
         # threw up its hands because of the `**request`.
         assert isinstance(model_response, Response)
 
+        # if this is a background request then poll for status until we get it
+        if background:
+            model_response = await wait_for_background_response(client, model_response)
+
         # check for error
         if model_response.error is not None:
             raise OpenAIResponseError(
@@ -132,6 +155,42 @@ async def generate_responses(
         ), model_call()
 
 
+async def wait_for_background_response(
+    client: AsyncAzureOpenAI | AsyncOpenAI, model_response: Response
+) -> Response:
+    # do some retrying so we don't waste expensive background work
+    # because of transient networking issues
+    @retry(
+        wait=wait_exponential_jitter(),
+        stop=stop_after_attempt(5) | stop_after_delay(60),
+        retry=retry_if_exception(httpx_should_retry),
+        before_sleep=log_httpx_retry_attempt(
+            f"background polling: {model_response.model}"
+        ),
+    )
+    async def check_model_response(model_response: Response) -> Response:
+        return await client.responses.retrieve(model_response.id)
+
+    try:
+        # keep checking status until we get "complete", "incomplete", of "failed"
+        while model_response.status in {"queued", "in_progress"}:
+            await anyio.sleep(5)
+            model_response = await check_model_response(model_response)
+        return model_response
+    except anyio.get_cancelled_exc_class():
+        # if the entire sample is cancelled then let the provider know
+        # so we can stop racking up token costs
+        with anyio.move_on_after(5, shield=True):
+            try:
+                await client.responses.cancel(model_response.id)
+            except BaseException as ex:
+                logger.warning(
+                    f"Error while attempting to cancel background request: {ex}"
+                )
+                pass
+        raise
+
+
 def completion_params_responses(
     model_name: str,
     *,
@@ -164,10 +223,10 @@ def completion_params_responses(
     if config.seed is not None:
         unsupported_warning("seed")
     if config.temperature is not None:
-        if openai_api.is_o_series():
+        if openai_api.is_o_series() or openai_api.is_gpt_5():
             warn_once(
                 logger,
-                "o series models do not support the 'temperature' parameter (temperature is always 1).",
+                "gpt-5 and o-series models do not support the 'temperature' parameter (temperature is always 1).",
             )
         else:
             params["temperature"] = config.temperature
@@ -185,7 +244,12 @@ def completion_params_responses(
         and not openai_api.is_o_series()
     ):
         params["parallel_tool_calls"] = config.parallel_tool_calls
-    if openai_api.is_o_series() and not openai_api.is_o1_early():
+
+    if (
+        (openai_api.is_o_series() and not openai_api.is_o1_early())
+        or openai_api.is_gpt_5()
+        or openai_api.is_codex()
+    ):
         reasoning: dict[str, str] = {}
         if config.reasoning_effort is not None:
             reasoning["effort"] = config.reasoning_effort
