@@ -1,6 +1,11 @@
 from ._types import MCPServer
 from .connection import mcp_connection
-from .server import mcp_server_sandbox, mcp_server_sse, mcp_server_stdio
+from .server import (
+    mcp_server_http,
+    mcp_server_sandbox,
+    mcp_server_sse,
+    mcp_server_stdio,
+)
 from .tools import mcp_tools
 
 __all__ = [
@@ -8,6 +13,7 @@ __all__ = [
     "mcp_server_stdio",
     "mcp_server_sse",
     "mcp_server_sandbox",
+    "mcp_server_http",
     "mcp_connection",
     "MCPServer",
 ]
