@@ -98,7 +98,10 @@ def check_tools_calls(model: Model, **model_args) -> None:
     model = get_model(model)
     task = Task(
         dataset=addition_dataset,
-        solver=[use_tools(addition()), generate()],
+        solver=[
+            use_tools(addition(), tool_choice=ToolFunction("addition")),
+            generate(),
+        ],
         scorer=match("any", numeric=True),
     )
 
@@ -172,7 +175,7 @@ def test_openai_responses_tools():
 
 @skip_if_no_anthropic
 def test_anthropic_tools():
-    check_tools("anthropic/claude-3-sonnet-20240229")
+    check_tools("anthropic/claude-3-7-sonnet-latest")
 
 
 @skip_if_no_mistral
